@@ -8,17 +8,46 @@ Application Base Class.
 :license: BSD, see LICENSE for more details.
 
 """
-import platform as _platform
+from __future__ import absolute_import
+from __future__ import with_statement
 
+import platform as _platform
+import sys
+
+from contextlib import contextmanager
 from copy import deepcopy
+from functools import wraps
 from threading import Lock
 
 from kombu.utils import cached_property
 
+from celery import datastructures
 from celery.app.defaults import DEFAULTS
-from celery.datastructures import ConfigurationView
 from celery.utils import instantiate, lpmerge
-from celery.utils.functional import wraps
+
+import kombu
+if kombu.VERSION < (1, 1, 0):
+    raise ImportError("Celery requires Kombu version 1.1.0 or higher.")
+
+BUGREPORT_INFO = """
+platform -> system:%(system)s arch:%(arch)s imp:%(py_i)s
+software -> celery:%(celery_v)s kombu:%(kombu_v)s py:%(py_v)s
+settings -> transport:%(transport)s results:%(results)s
+"""
+
+
+def pyimplementation():
+    if hasattr(_platform, "python_implementation"):
+        return _platform.python_implementation()
+    elif sys.platform.startswith("java"):
+        return "Jython %s" % (sys.platform, )
+    elif hasattr(sys, "pypy_version_info"):
+        v = ".".join(map(str, sys.pypy_version_info[:3]))
+        if sys.pypy_version_info[3:]:
+            v += "-" + "".join(map(str, sys.pypy_version_info[3:]))
+        return "PyPy %s" % (v, )
+    else:
+        return "CPython"
 
 
 class LamportClock(object):
@@ -59,24 +88,39 @@ class LamportClock(object):
 
     def __init__(self, initial_value=0):
         self.value = initial_value
-        self._mutex = Lock()
+        self.mutex = Lock()
 
     def adjust(self, other):
-        self._mutex.acquire()
-        try:
+        with self.mutex:
             self.value = max(self.value, other) + 1
-        finally:
-            self._mutex.release()
 
     def forward(self):
-        self._mutex.acquire()
-        try:
+        with self.mutex:
             self.value += 1
-        finally:
-            self._mutex.release()
         return self.value
 
 
+class Settings(datastructures.ConfigurationView):
+
+    @property
+    def CELERY_RESULT_BACKEND(self):
+        """Resolves deprecated alias ``CELERY_BACKEND``."""
+        return self.get("CELERY_RESULT_BACKEND") or self.get("CELERY_BACKEND")
+
+    @property
+    def BROKER_TRANSPORT(self):
+        """Resolves compat aliases :setting:`BROKER_BACKEND`
+        and :setting:`CARROT_BACKEND`."""
+        return (self.get("BROKER_TRANSPORT") or
+                self.get("BROKER_BACKEND") or
+                self.get("CARROT_BACKEND"))
+
+    @property
+    def BROKER_BACKEND(self):
+        """Deprecated compat alias to :attr:`BROKER_TRANSPORT`."""
+        return self.BROKER_TRANSPORT
+
+
 class BaseApp(object):
     """Base class for apps."""
     SYSTEM = _platform.system()
@@ -86,10 +130,12 @@ class BaseApp(object):
     amqp_cls = "celery.app.amqp.AMQP"
     backend_cls = None
     events_cls = "celery.events.Events"
-    loader_cls = "app"
+    loader_cls = "celery.loaders.app.AppLoader"
     log_cls = "celery.log.Logging"
     control_cls = "celery.task.control.Control"
 
+    _pool = None
+
     def __init__(self, main=None, loader=None, backend=None,
             amqp=None, events=None, log=None, control=None,
             set_as_current=True, accept_magic_kwargs=False):
@@ -102,16 +148,17 @@ class BaseApp(object):
         self.control_cls = control or self.control_cls
         self.set_as_current = set_as_current
         self.accept_magic_kwargs = accept_magic_kwargs
-        self.on_init()
         self.clock = LamportClock()
 
+        self.on_init()
+
     def on_init(self):
         """Called at the end of the constructor."""
         pass
 
     def config_from_object(self, obj, silent=False):
         """Read configuration from object, where object is either
-        a real object, or the name of an object to import.
+        a object, or the name of a module to import.
 
             >>> celery.config_from_object("myapp.celeryconfig")
 
@@ -126,7 +173,7 @@ class BaseApp(object):
         """Read configuration from environment variable.
 
         The value of the environment variable must be the name
-        of an object to import.
+        of a module to import.
 
             >>> os.environ["CELERY_CONFIG_MODULE"] = "myapp.celeryconfig"
             >>> celery.config_from_envvar("CELERY_CONFIG_MODULE")
@@ -154,7 +201,7 @@ class BaseApp(object):
             using :meth:`AsyncResult`.
 
         Supports the same arguments as
-        :meth:`~celery.task.base.BaseTask.apply_async`.
+        :meth:`~celery.app.task.BaseTask.apply_async`.
 
         """
         router = self.amqp.Router(queues)
@@ -166,8 +213,8 @@ class BaseApp(object):
         exchange = options.get("exchange")
         exchange_type = options.get("exchange_type")
 
-        def _do_publish(connection=None, **_):
-            publish = publisher or self.amqp.TaskPublisher(connection,
+        with self.default_connection(connection, connect_timeout) as conn:
+            publish = publisher or self.amqp.TaskPublisher(conn,
                                             exchange=exchange,
                                             exchange_type=exchange_type)
             try:
@@ -177,17 +224,12 @@ class BaseApp(object):
                                             expires=expires, **options)
             finally:
                 publisher or publish.close()
-
             return result_cls(new_id)
 
-        return self.with_default_connection(_do_publish)(
-                connection=connection, connect_timeout=connect_timeout)
-
     def AsyncResult(self, task_id, backend=None, task_name=None):
         """Create :class:`celery.result.BaseAsyncResult` instance."""
         from celery.result import BaseAsyncResult
-        return BaseAsyncResult(task_id, app=self,
-                               task_name=task_name,
+        return BaseAsyncResult(task_id, app=self, task_name=task_name,
                                backend=backend or self.backend)
 
     def TaskSetResult(self, taskset_id, results, **kwargs):
@@ -209,7 +251,7 @@ class BaseApp(object):
         :keyword insist: defaults to the :setting:`BROKER_INSIST` setting.
         :keyword connect_timeout: defaults to the
             :setting:`BROKER_CONNECTION_TIMEOUT` setting.
-        :keyword backend_cls: defaults to the :setting:`BROKER_BACKEND`
+        :keyword backend_cls: defaults to the :setting:`BROKER_TRANSPORT`
             setting.
 
         :returns :class:`kombu.connection.BrokerConnection`:
@@ -221,13 +263,29 @@ class BaseApp(object):
                     password or self.conf.BROKER_PASSWORD,
                     virtual_host or self.conf.BROKER_VHOST,
                     port or self.conf.BROKER_PORT,
-                    transport=transport or self.conf.BROKER_BACKEND,
+                    transport=transport or self.conf.BROKER_TRANSPORT,
                     insist=self.either("BROKER_INSIST", insist),
                     ssl=self.either("BROKER_USE_SSL", ssl),
                     connect_timeout=self.either(
                                 "BROKER_CONNECTION_TIMEOUT", connect_timeout),
                     transport_options=self.conf.BROKER_TRANSPORT_OPTIONS)
 
+    @contextmanager
+    def default_connection(self, connection=None, connect_timeout=None):
+        """For use within a with-statement to get a connection from the pool
+        if one is not already provided.
+
+        :keyword connection: If not provided, then a connection will be
+                             acquired from the connection pool.
+        :keyword connect_timeout: *No longer used.*
+
+        """
+        if connection:
+            yield connection
+        else:
+            with self.pool.acquire(block=True) as connection:
+                yield connection
+
     def with_default_connection(self, fun):
         """With any function accepting `connection` and `connect_timeout`
         keyword arguments, establishes a default connection if one is
@@ -236,37 +294,24 @@ class BaseApp(object):
         Any automatically established connection will be closed after
         the function returns.
 
-        """
+        **Deprecated**
 
+        Use ``with app.default_connection(connection)`` instead.
+
+        """
         @wraps(fun)
         def _inner(*args, **kwargs):
-            connection = kwargs.get("connection")
-            timeout = kwargs.get("connect_timeout")
-            kwargs["connection"] = conn = connection or \
-                    self.broker_connection(connect_timeout=timeout)
-            close_connection = not connection and conn.close or None
-
-            try:
-                return fun(*args, **kwargs)
-            finally:
-                if close_connection:
-                    close_connection()
+            connection = kwargs.pop("connection", None)
+            with self.default_connection(connection) as c:
+                return fun(*args, **dict(kwargs, connection=c))
         return _inner
 
     def prepare_config(self, c):
         """Prepare configuration before it is merged with the defaults."""
-        if not c.get("CELERY_RESULT_BACKEND"):
-            rbackend = c.get("CELERY_BACKEND")
-            if rbackend:
-                c["CELERY_RESULT_BACKEND"] = rbackend
-        if not c.get("BROKER_BACKEND"):
-            cbackend = c.get("BROKER_TRANSPORT") or c.get("CARROT_BACKEND")
-            if cbackend:
-                c["BROKER_BACKEND"] = cbackend
         return c
 
     def mail_admins(self, subject, body, fail_silently=False):
-        """Send an e-mail to the admins in the :setting:`ADMINS` setting."""
+        """Send an email to the admins in the :setting:`ADMINS` setting."""
         if self.conf.ADMINS:
             to = [admin_email for _, admin_email in self.conf.ADMINS]
             return self.loader.mail_admins(subject, body, fail_silently, to=to,
@@ -275,7 +320,8 @@ class BaseApp(object):
                                        port=self.conf.EMAIL_PORT,
                                        user=self.conf.EMAIL_HOST_USER,
                                        password=self.conf.EMAIL_HOST_PASSWORD,
-                                       timeout=self.conf.EMAIL_TIMEOUT)
+                                       timeout=self.conf.EMAIL_TIMEOUT,
+                                       use_ssl=self.conf.EMAIL_USE_SSL)
 
     def either(self, default_key, *values):
         """Fallback to the value of a configuration key if none of the
@@ -297,8 +343,37 @@ class BaseApp(object):
         return backend_cls(app=self)
 
     def _get_config(self):
-        return ConfigurationView({},
-                [self.prepare_config(self.loader.conf), deepcopy(DEFAULTS)])
+        return Settings({}, [self.prepare_config(self.loader.conf),
+                             deepcopy(DEFAULTS)])
+
+    def _after_fork(self, obj_):
+        if self._pool:
+            self._pool.force_close_all()
+            self._pool = None
+
+    def bugreport(self):
+        import celery
+        import kombu
+        return BUGREPORT_INFO % {"system": _platform.system(),
+                                 "arch": _platform.architecture(),
+                                 "py_i": pyimplementation(),
+                                 "celery_v": celery.__version__,
+                                 "kombu_v": kombu.__version__,
+                                 "py_v": _platform.python_version(),
+                                 "transport": self.conf.BROKER_TRANSPORT,
+                                 "results": self.conf.CELERY_RESULT_BACKEND}
+
+    @property
+    def pool(self):
+        if self._pool is None:
+            try:
+                from multiprocessing.util import register_after_fork
+                register_after_fork(self, self._after_fork)
+            except ImportError:
+                pass
+            limit = self.conf.BROKER_POOL_LIMIT
+            self._pool = self.broker_connection().Pool(limit)
+        return self._pool
 
     @cached_property
     def amqp(self):
