@@ -1,8 +1,12 @@
+from __future__ import absolute_import
+from __future__ import with_statement
+
 import time
 import socket
 import threading
 
 from collections import deque
+from contextlib import contextmanager
 from itertools import count
 
 from kombu.entity import Exchange, Queue
@@ -14,18 +18,16 @@ from celery.utils import gen_unique_id
 event_exchange = Exchange("celeryev", type="topic")
 
 
-def create_event(type, fields):
-    return dict(fields, type=type,
-                        timestamp=fields.get("timestamp") or time.time())
-
-
-def Event(type, **fields):
+def Event(type, _fields=None, **fields):
     """Create an event.
 
-    An event is a dictionary, the only required field is the type.
+    An event is a dictionary, the only required field is ``type``.
 
     """
-    return create_event(type, fields)
+    event = dict(_fields or {}, type=type, **fields)
+    if "timestamp" not in event:
+        event["timestamp"] = time.time()
+    return event
 
 
 class EventDispatcher(object):
@@ -57,28 +59,32 @@ class EventDispatcher(object):
         self.connection = connection
         self.channel = channel
         self.hostname = hostname or socket.gethostname()
-        self.enabled = enabled
         self.buffer_while_offline = buffer_while_offline
-        self._lock = threading.Lock()
+        self.mutex = threading.Lock()
         self.publisher = None
         self._outbound_buffer = deque()
         self.serializer = serializer or self.app.conf.CELERY_EVENT_SERIALIZER
 
+        self.enabled = enabled
         if self.enabled:
             self.enable()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *exc_info):
+        self.close()
+
     def enable(self):
-        self.enabled = True
-        channel = self.channel or self.connection.channel()
-        self.publisher = Producer(channel, exchange=event_exchange,
+        self.publisher = Producer(self.channel or self.connection.channel(),
+                                  exchange=event_exchange,
                                   serializer=self.serializer)
+        self.enabled = True
 
     def disable(self):
-        self.enabled = False
-        if self.publisher is not None:
-            if not self.channel:  # close auto channel.
-                self.publisher.channel.close()
-            self.publisher = None
+        if self.enabled:
+            self.enabled = False
+            self.close()
 
     def send(self, type, **fields):
         """Send event.
@@ -87,22 +93,17 @@ class EventDispatcher(object):
         :keyword \*\*fields: Event arguments.
 
         """
-        if not self.enabled:
-            return
-
-        self._lock.acquire()
-        event = Event(type, hostname=self.hostname,
-                            clock=self.app.clock.forward(), **fields)
-        try:
-            try:
-                self.publisher.publish(event,
-                                       routing_key=type.replace("-", "."))
-            except Exception, exc:
-                if not self.buffer_while_offline:
-                    raise
-                self._outbound_buffer.append((type, fields, exc))
-        finally:
-            self._lock.release()
+        if self.enabled:
+            with self.mutex:
+                event = Event(type, hostname=self.hostname,
+                                    clock=self.app.clock.forward(), **fields)
+                try:
+                    self.publisher.publish(event,
+                                           routing_key=type.replace("-", "."))
+                except Exception, exc:
+                    if not self.buffer_while_offline:
+                        raise
+                    self._outbound_buffer.append((type, fields, exc))
 
     def flush(self):
         while self._outbound_buffer:
@@ -117,8 +118,11 @@ class EventDispatcher(object):
 
     def close(self):
         """Close the event dispatcher."""
-        self._lock.locked() and self._lock.release()
-        self.publisher and self.publisher.channel.close()
+        self.mutex.locked() and self.mutex.release()
+        if self.publisher is not None:
+            if not self.channel:  # close auto channel.
+                self.publisher.channel.close()
+            self.publisher = None
 
 
 class EventReceiver(object):
@@ -154,6 +158,7 @@ class EventReceiver(object):
         handler = self.handlers.get(type) or self.handlers.get("*")
         handler and handler(event)
 
+    @contextmanager
     def consumer(self):
         """Create event consumer.
 
@@ -164,24 +169,20 @@ class EventReceiver(object):
 
         """
         consumer = Consumer(self.connection.channel(),
-                            queues=[self.queue],
-                            no_ack=True)
+                            queues=[self.queue], no_ack=True)
         consumer.register_callback(self._receive)
-        return consumer
+        with consumer:
+            yield consumer
+        consumer.channel.close()
 
     def itercapture(self, limit=None, timeout=None, wakeup=True):
-        consumer = self.consumer()
-        consumer.consume()
-        if wakeup:
-            self.wakeup_workers(channel=consumer.channel)
+        with self.consumer() as consumer:
+            if wakeup:
+                self.wakeup_workers(channel=consumer.channel)
 
-        yield consumer
+            yield consumer
 
-        try:
             self.drain_events(limit=limit, timeout=timeout)
-        finally:
-            consumer.cancel()
-            consumer.channel.close()
 
     def capture(self, limit=None, timeout=None, wakeup=True):
         """Open up a consumer capturing events.
@@ -190,9 +191,7 @@ class EventReceiver(object):
         stop unless forced via :exc:`KeyboardInterrupt` or :exc:`SystemExit`.
 
         """
-        list(self.itercapture(limit=limit,
-                              timeout=timeout,
-                              wakeup=wakeup))
+        list(self.itercapture(limit=limit, timeout=timeout, wakeup=wakeup))
 
     def wakeup_workers(self, channel=None):
         self.app.control.broadcast("heartbeat",
@@ -216,7 +215,7 @@ class EventReceiver(object):
         clock = body.get("clock")
         if clock:
             self.app.clock.adjust(clock)
-        self.process(type, create_event(type, body))
+        self.process(type, Event(type, body))
 
 
 class Events(object):
@@ -243,3 +242,11 @@ class Events(object):
     def State(self):
         from celery.events.state import State as _State
         return _State()
+
+    @contextmanager
+    def default_dispatcher(self, hostname=None, enabled=True,
+            buffer_while_offline=False):
+        with self.app.amqp.publisher_pool.acquire(block=True) as pub:
+            with self.Dispatcher(pub.connection, hostname, enabled,
+                                 pub.channel, buffer_while_offline) as d:
+                yield d
