@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+
 import importlib
 import os
 import re
@@ -8,9 +10,10 @@ from kombu.utils import cached_property
 
 from celery.datastructures import DictAttribute
 from celery.exceptions import ImproperlyConfigured
+from celery.utils import get_cls_by_name
 from celery.utils import import_from_cwd as _import_from_cwd
 
-BUILTIN_MODULES = ["celery.task"]
+BUILTIN_MODULES = frozenset(["celery.task"])
 
 ERROR_ENVVAR_NOT_SET = (
 """The environment variable %r is not set,
@@ -22,7 +25,7 @@ a configuration module.""")
 class BaseLoader(object):
     """The base class for loaders.
 
-    Loaders handles to following things:
+    Loaders handles,
 
         * Reading celery client/worker configurations.
 
@@ -35,9 +38,12 @@ class BaseLoader(object):
         * What modules are imported to find tasks?
 
     """
-    worker_initialized = False
-    override_backends = {}
+    builtin_modules = BUILTIN_MODULES
     configured = False
+    error_envvar_not_set = ERROR_ENVVAR_NOT_SET
+    override_backends = {}
+    worker_initialized = False
+
     _conf = None
 
     def __init__(self, app=None, **kwargs):
@@ -64,14 +70,13 @@ class BaseLoader(object):
         return importlib.import_module(module)
 
     def import_from_cwd(self, module, imp=None):
-        if imp is None:
-            imp = self.import_module
-        return _import_from_cwd(module, imp)
+        return _import_from_cwd(module,
+                self.import_module if imp is None else imp)
 
     def import_default_modules(self):
-        imports = self.conf.get("CELERY_IMPORTS") or []
-        imports = set(list(imports) + BUILTIN_MODULES)
-        return map(self.import_task_module, imports)
+        imports = set(list(self.conf.get("CELERY_IMPORTS") or ()))
+        return [self.import_task_module(module)
+                    for module in imports | self.builtin_modules]
 
     def init_worker(self):
         if not self.worker_initialized:
@@ -83,13 +88,16 @@ class BaseLoader(object):
         if not module_name:
             if silent:
                 return False
-            raise ImproperlyConfigured(ERROR_ENVVAR_NOT_SET % (module_name, ))
+            raise ImproperlyConfigured(self.error_envvar_not_set % module_name)
         return self.config_from_object(module_name, silent=silent)
 
     def config_from_object(self, obj, silent=False):
         if isinstance(obj, basestring):
             try:
-                obj = self.import_from_cwd(obj)
+                if "." in obj:
+                    obj = get_cls_by_name(obj, imp=self.import_from_cwd)
+                else:
+                    obj = self.import_from_cwd(obj)
             except (ImportError, AttributeError):
                 if silent:
                     return False
@@ -147,13 +155,13 @@ class BaseLoader(object):
 
     def mail_admins(self, subject, body, fail_silently=False,
             sender=None, to=None, host=None, port=None,
-            user=None, password=None, timeout=None):
+            user=None, password=None, timeout=None, use_ssl=False):
         try:
             message = self.mail.Message(sender=sender, to=to,
                                         subject=subject, body=body)
             mailer = self.mail.Mailer(host=host, port=port,
                                       user=user, password=password,
-                                      timeout=timeout)
+                                      timeout=timeout, use_ssl=use_ssl)
             mailer.send(message)
         except Exception, exc:
             if not fail_silently:
@@ -171,5 +179,4 @@ class BaseLoader(object):
 
     @cached_property
     def mail(self):
-        from celery.utils import mail
-        return mail
+        return self.import_module("celery.utils.mail")
