@@ -1,17 +1,24 @@
+from __future__ import with_statement
+
 import sys
 import types
-import unittest2 as unittest
 
-from celery.serialization import subclass_exception
-from celery.serialization import find_nearest_pickleable_exception as fnpe
-from celery.serialization import UnpickleableExceptionWrapper
-from celery.serialization import get_pickleable_exception as gpe
+from mock import Mock
+
+from celery.utils import serialization
+from celery.utils.serialization import subclass_exception
+from celery.utils.serialization import \
+        find_nearest_pickleable_exception as fnpe
+from celery.utils.serialization import UnpickleableExceptionWrapper
+from celery.utils.serialization import get_pickleable_exception as gpe
 
 from celery import states
 from celery.backends.base import BaseBackend, KeyValueStoreBackend
-from celery.backends.base import BaseDictBackend
+from celery.backends.base import BaseDictBackend, DisabledBackend
 from celery.utils import gen_unique_id
 
+from celery.tests.utils import unittest
+
 
 class wrapobject(object):
 
@@ -26,16 +33,33 @@ Lookalike = subclass_exception("Lookalike", wrapobject, "foo.module")
 b = BaseBackend()
 
 
+class test_serialization(unittest.TestCase):
+
+    def test_create_exception_cls(self):
+        self.assertTrue(serialization.create_exception_cls("FooError", "m"))
+        self.assertTrue(serialization.create_exception_cls("FooError",
+                                                            "m",
+                                                            KeyError))
+
+
 class test_BaseBackend_interface(unittest.TestCase):
 
     def test_get_status(self):
         self.assertRaises(NotImplementedError,
                 b.get_status, "SOMExx-N0Nex1stant-IDxx-")
 
+    def test__forget(self):
+        self.assertRaises(NotImplementedError,
+                b.forget, "SOMExx-N0Nex1stant-IDxx-")
+
     def test_store_result(self):
         self.assertRaises(NotImplementedError,
                 b.store_result, "SOMExx-N0nex1stant-IDxx-", 42, states.SUCCESS)
 
+    def test_mark_as_started(self):
+        self.assertRaises(NotImplementedError,
+                b.mark_as_started, "SOMExx-N0nex1stant-IDxx-")
+
     def test_reload_task_result(self):
         self.assertRaises(NotImplementedError,
                 b.reload_task_result, "SOMExx-N0nex1stant-IDxx-")
@@ -52,6 +76,10 @@ class test_BaseBackend_interface(unittest.TestCase):
         self.assertRaises(NotImplementedError,
                 b.restore_taskset, "SOMExx-N0nex1stant-IDxx-")
 
+    def test_delete_taskset(self):
+        self.assertRaises(NotImplementedError,
+                b.delete_taskset, "SOMExx-N0nex1stant-IDxx-")
+
     def test_save_taskset(self):
         self.assertRaises(NotImplementedError,
                 b.save_taskset, "SOMExx-N0nex1stant-IDxx-", "blergh")
@@ -60,6 +88,19 @@ class test_BaseBackend_interface(unittest.TestCase):
         self.assertRaises(NotImplementedError,
                 b.get_traceback, "SOMExx-N0nex1stant-IDxx-")
 
+    def test_forget(self):
+        self.assertRaises(NotImplementedError,
+                b.forget, "SOMExx-N0nex1stant-IDxx-")
+
+    def test_on_chord_apply(self, unlock="celery.chord_unlock"):
+        from celery.registry import tasks
+        p, tasks[unlock] = tasks.get(unlock), Mock()
+        try:
+            b.on_chord_apply("dakj221", "sdokqweok")
+            self.assertTrue(tasks[unlock].apply_async.call_count)
+        finally:
+            tasks[unlock] = p
+
 
 class test_exception_pickle(unittest.TestCase):
 
@@ -104,6 +145,7 @@ class test_prepare_exception(unittest.TestCase):
 
 
 class KVBackend(KeyValueStoreBackend):
+    mget_returns_dict = False
 
     def __init__(self, *args, **kwargs):
         self.db = {}
@@ -115,11 +157,21 @@ class KVBackend(KeyValueStoreBackend):
     def set(self, key, value):
         self.db[key] = value
 
+    def mget(self, keys):
+        if self.mget_returns_dict:
+            return dict((key, self.get(key)) for key in keys)
+        else:
+            return [self.get(key) for key in keys]
+
+    def delete(self, key):
+        self.db.pop(key, None)
+
 
 class DictBackend(BaseDictBackend):
 
-    def _save_taskset(self, taskset_id, result):
-        return "taskset-saved"
+    def __init__(self, *args, **kwargs):
+        BaseDictBackend.__init__(self, *args, **kwargs)
+        self._data = {"can-delete": {"result": "foo"}}
 
     def _restore_taskset(self, taskset_id):
         if taskset_id == "exists":
@@ -129,15 +181,29 @@ class DictBackend(BaseDictBackend):
         if task_id == "task-exists":
             return {"result": "task"}
 
+    def _delete_taskset(self, taskset_id):
+        self._data.pop(taskset_id, None)
+
 
 class test_BaseDictBackend(unittest.TestCase):
 
     def setUp(self):
         self.b = DictBackend()
 
+    def test_delete_taskset(self):
+        self.b.delete_taskset("can-delete")
+        self.assertNotIn("can-delete", self.b._data)
+
     def test_save_taskset(self):
-        self.assertEqual(self.b.save_taskset("foofoo", "xxx"),
-                         "taskset-saved")
+        b = BaseDictBackend()
+        b._save_taskset = Mock()
+        b.save_taskset("foofoo", "xxx")
+        b._save_taskset.assert_called_with("foofoo", "xxx")
+
+    def test_forget_interface(self):
+        b = BaseDictBackend()
+        with self.assertRaises(NotImplementedError):
+            b.forget("foo")
 
     def test_restore_taskset(self):
         self.assertIsNone(self.b.restore_taskset("missing"))
@@ -154,7 +220,7 @@ class test_BaseDictBackend(unittest.TestCase):
 
     def test_reload_task_result(self):
         self.b._cache = {}
-        self.b.reload_taskset_result("task-exists")
+        self.b.reload_task_result("task-exists")
         self.b._cache["task-exists"] = {"result": "task"}
 
 
@@ -163,20 +229,41 @@ class test_KeyValueStoreBackend(unittest.TestCase):
     def setUp(self):
         self.b = KVBackend()
 
-    def test_get_store_result(self):
+    def test_get_store_delete_result(self):
         tid = gen_unique_id()
         self.b.mark_as_done(tid, "Hello world")
         self.assertEqual(self.b.get_result(tid), "Hello world")
         self.assertEqual(self.b.get_status(tid), states.SUCCESS)
+        self.b.forget(tid)
+        self.assertEqual(self.b.get_status(tid), states.PENDING)
+
+    def test_strip_prefix(self):
+        x = self.b.get_key_for_task("x1b34")
+        self.assertEqual(self.b._strip_prefix(x), "x1b34")
+        self.assertEqual(self.b._strip_prefix("x1b34"), "x1b34")
+
+    def test_get_many(self):
+        for is_dict in True, False:
+            self.b.mget_returns_dict = is_dict
+            ids = dict((gen_unique_id(), i) for i in xrange(10))
+            for id, i in ids.items():
+                self.b.mark_as_done(id, i)
+            it = self.b.get_many(ids.keys())
+            for i, (got_id, got_state) in enumerate(it):
+                self.assertEqual(got_state["result"], ids[got_id])
+            self.assertEqual(i, 9)
+            self.assertTrue(list(self.b.get_many(ids.keys())))
 
     def test_get_missing_meta(self):
         self.assertIsNone(self.b.get_result("xxx-missing"))
         self.assertEqual(self.b.get_status("xxx-missing"), states.PENDING)
 
-    def test_save_restore_taskset(self):
+    def test_save_restore_delete_taskset(self):
         tid = gen_unique_id()
         self.b.save_taskset(tid, "Hello world")
         self.assertEqual(self.b.restore_taskset(tid), "Hello world")
+        self.b.delete_taskset(tid)
+        self.assertIsNone(self.b.restore_taskset(tid))
 
     def test_restore_missing_taskset(self):
         self.assertIsNone(self.b.restore_taskset("xxx-nonexistant"))
@@ -194,3 +281,25 @@ class test_KeyValueStoreBackend_interface(unittest.TestCase):
 
     def test_cleanup(self):
         self.assertFalse(KeyValueStoreBackend().cleanup())
+
+    def test_delete(self):
+        self.assertRaises(NotImplementedError, KeyValueStoreBackend().delete,
+                "a")
+
+    def test_mget(self):
+        self.assertRaises(NotImplementedError, KeyValueStoreBackend().mget,
+                ["a"])
+
+    def test_forget(self):
+        self.assertRaises(NotImplementedError, KeyValueStoreBackend().forget,
+                "a")
+
+
+class test_DisabledBackend(unittest.TestCase):
+
+    def test_store_result(self):
+        DisabledBackend().store_result()
+
+    def test_is_disabled(self):
+        with self.assertRaises(NotImplementedError):
+            DisabledBackend().get_status("foo")
