@@ -1,14 +1,17 @@
+from __future__ import with_statement
+
 import logging
 import os
 import sys
 import warnings
 
+from functools import wraps
 try:
     from multiprocessing import current_process
 except ImportError:
-    current_process = None
-
+    current_process = None  # noqa
 
+from mock import patch
 from nose import SkipTest
 from kombu.tests.utils import redirect_stdouts
 
@@ -20,16 +23,14 @@ from celery.apps import worker as cd
 from celery.bin.celeryd import WorkerCommand, windows_main, \
                                main as celeryd_main
 from celery.exceptions import ImproperlyConfigured
-from celery.utils import patch
-from celery.utils.functional import wraps
 
 from celery.tests.compat import catch_warnings
-from celery.tests.utils import execute_context
-from celery.tests.utils import AppCase
-from celery.tests.utils import StringIO
+from celery.tests.utils import (AppCase, StringIO, mask_modules,
+                                reset_modules, skip_unless_module)
 
 
-patch.ensure_process_aware_logger()
+from celery.utils.patch import ensure_process_aware_logger
+ensure_process_aware_logger()
 
 
 def disable_stdouts(fun):
@@ -59,6 +60,49 @@ class Worker(cd.Worker):
     WorkController = _WorkController
 
 
+class test_compilation(AppCase):
+
+    def test_no_multiprocessing(self):
+        with mask_modules("multiprocessing"):
+            with reset_modules("celery.apps.worker"):
+                from celery.apps.worker import multiprocessing
+                self.assertIsNone(multiprocessing)
+
+    def test_cpu_count_no_mp(self):
+        with mask_modules("multiprocessing"):
+            with reset_modules("celery.apps.worker"):
+                from celery.apps.worker import cpu_count
+                self.assertEqual(cpu_count(), 2)
+
+    @skip_unless_module("multiprocessing")
+    def test_no_cpu_count(self):
+
+        @patch("multiprocessing.cpu_count")
+        def _do_test(pcount):
+            pcount.side_effect = NotImplementedError("cpu_count")
+            from celery.apps.worker import cpu_count
+            self.assertEqual(cpu_count(), 2)
+            pcount.assert_called_with()
+
+        _do_test()
+
+    def test_process_name_wo_mp(self):
+        with mask_modules("multiprocessing"):
+            with reset_modules("celery.apps.worker"):
+                from celery.apps.worker import get_process_name
+                self.assertIsNone(get_process_name())
+
+    @skip_unless_module("multiprocessing")
+    def test_process_name_w_mp(self):
+
+        @patch("multiprocessing.current_process")
+        def _do_test(current_process):
+            from celery.apps.worker import get_process_name
+            self.assertTrue(get_process_name())
+
+        _do_test()
+
+
 class test_Worker(AppCase):
     Worker = Worker
 
@@ -70,6 +114,27 @@ class test_Worker(AppCase):
         self.assertEqual(worker.use_queues, ["foo", "bar", "baz"])
         self.assertTrue("foo" in celery.amqp.queues)
 
+    @disable_stdouts
+    def test_windows_B_option(self):
+        celery = Celery(set_as_current=False)
+        celery.IS_WINDOWS = True
+        with self.assertRaises(SystemExit):
+            celery.Worker(run_clockservice=True)
+
+    def test_tasklist(self):
+        celery = Celery(set_as_current=False)
+        worker = celery.Worker()
+        self.assertTrue(worker.tasklist(include_builtins=True))
+        worker.tasklist(include_builtins=False)
+
+    def test_extra_info(self):
+        celery = Celery(set_as_current=False)
+        worker = celery.Worker()
+        worker.loglevel = logging.WARNING
+        self.assertFalse(worker.extra_info())
+        worker.loglevel = logging.INFO
+        self.assertTrue(worker.extra_info())
+
     @disable_stdouts
     def test_loglevel_string(self):
         worker = self.Worker(loglevel="INFO")
@@ -78,11 +143,13 @@ class test_Worker(AppCase):
     def test_run_worker(self):
         handlers = {}
 
-        def i(sig, handler):
-            handlers[sig] = handler
+        class Signals(platforms.Signals):
 
-        p = platforms.install_signal_handler
-        platforms.install_signal_handler = i
+            def __setitem__(self, sig, handler):
+                handlers[sig] = handler
+
+        p = platforms.signals
+        platforms.signals = Signals()
         try:
             w = self.Worker()
             w._isatty = False
@@ -98,7 +165,7 @@ class test_Worker(AppCase):
                 self.assertIn(sig, handlers)
             self.assertNotIn("SIGHUP", handlers)
         finally:
-            platforms.install_signal_handler = p
+            platforms.signals = p
 
     @disable_stdouts
     def test_startup_info(self):
@@ -109,6 +176,8 @@ class test_Worker(AppCase):
         self.assertTrue(worker.startup_info())
         worker.loglevel = logging.INFO
         self.assertTrue(worker.startup_info())
+        worker.autoscale = 13, 10
+        self.assertTrue(worker.startup_info())
 
     @disable_stdouts
     def test_run(self):
@@ -182,21 +251,19 @@ class test_Worker(AppCase):
             raise SkipTest("Not applicable on Windows")
         warnings.resetwarnings()
 
-        def getuid():
+        def geteuid():
             return 0
 
-        prev, os.getuid = os.getuid, getuid
+        prev, os.geteuid = os.geteuid, geteuid
         try:
-            def with_catch_warnings(log):
+            with catch_warnings(record=True) as log:
                 worker = self.Worker()
                 worker.run()
                 self.assertTrue(log)
-                self.assertIn("superuser privileges is discouraged",
+                self.assertIn("superuser privileges is not encouraged",
                               log[0].message.args[0])
-            context = catch_warnings(record=True)
-            execute_context(context, with_catch_warnings)
         finally:
-            os.getuid = prev
+            os.geteuid = prev
 
     @disable_stdouts
     def test_use_pidfile(self):
@@ -379,16 +446,16 @@ class test_signal_handlers(AppCase):
     def psig(self, fun, *args, **kwargs):
         handlers = {}
 
-        def i(sig, handler):
-            handlers[sig] = handler
+        class Signals(platforms.Signals):
+            def __setitem__(self, sig, handler):
+                handlers[sig] = handler
 
-        p, platforms.install_signal_handler = \
-                platforms.install_signal_handler, i
+        p, platforms.signals = platforms.signals, Signals()
         try:
             fun(*args, **kwargs)
             return handlers
         finally:
-            platforms.install_signal_handler = p
+            platforms.signals = p
 
     @disable_stdouts
     def test_worker_int_handler(self):
@@ -396,17 +463,18 @@ class test_signal_handlers(AppCase):
         handlers = self.psig(cd.install_worker_int_handler, worker)
         next_handlers = {}
 
-        def i(sig, handler):
-            next_handlers[sig] = handler
+        class Signals(platforms.Signals):
 
-        p = platforms.install_signal_handler
-        platforms.install_signal_handler = i
+            def __setitem__(self, sig, handler):
+                next_handlers[sig] = handler
+
+        p, platforms.signals = platforms.signals, Signals()
         try:
             self.assertRaises(SystemExit, handlers["SIGINT"],
                               "SIGINT", object())
             self.assertTrue(worker.stopped)
         finally:
-            platforms.install_signal_handler = p
+            platforms.signals = p
 
         self.assertRaises(SystemExit, next_handlers["SIGINT"],
                           "SIGINT", object())
