@@ -1,7 +1,7 @@
-from celery.tests.utils import unittest
-from celery.tests.utils import StringIO
 from datetime import datetime, timedelta
+from functools import wraps
 
+from mock import Mock
 from pyparsing import ParseException
 
 from celery import task
@@ -11,12 +11,10 @@ from celery.exceptions import RetryTaskError
 from celery.execute import send_task
 from celery.result import EagerResult
 from celery.schedules import crontab, crontab_parser
-from celery.utils import timeutils
 from celery.utils import gen_unique_id
-from celery.utils.functional import wraps
 from celery.utils.timeutils import parse_iso8601
 
-from celery.tests.utils import with_eager_tasks
+from celery.tests.utils import with_eager_tasks, unittest, StringIO
 
 
 def return_True(*args, **kwargs):
@@ -62,15 +60,15 @@ class RetryTask(task.Task):
     max_retries = 3
     iterations = 0
 
-    def run(self, arg1, arg2, kwarg=1, **kwargs):
+    def run(self, arg1, arg2, kwarg=1, max_retries=None, care=True):
         self.__class__.iterations += 1
+        rmax = self.max_retries if max_retries is None else max_retries
 
-        retries = kwargs["task_retries"]
-        if retries >= 3:
+        retries = self.request.retries
+        if care and retries >= rmax:
             return arg1
         else:
-            kwargs.update({"kwarg": kwarg})
-            return self.retry(args=[arg1, arg2], kwargs=kwargs, countdown=0)
+            return self.retry(countdown=0, max_retries=max_retries)
 
 
 class RetryTaskNoArgs(task.Task):
@@ -139,6 +137,12 @@ class TestTaskRetries(unittest.TestCase):
         self.assertEqual(result.get(), 0xFF)
         self.assertEqual(RetryTask.iterations, 4)
 
+        RetryTask.max_retries = 3
+        RetryTask.iterations = 0
+        result = RetryTask.apply([0xFF, 0xFFFF], {"max_retries": 10})
+        self.assertEqual(result.get(), 0xFF)
+        self.assertEqual(RetryTask.iterations, 11)
+
     def test_retry_no_args(self):
         RetryTaskNoArgs.max_retries = 3
         RetryTaskNoArgs.iterations = 0
@@ -185,14 +189,14 @@ class TestTaskRetries(unittest.TestCase):
     def test_max_retries_exceeded(self):
         RetryTask.max_retries = 2
         RetryTask.iterations = 0
-        result = RetryTask.apply([0xFF, 0xFFFF])
+        result = RetryTask.apply([0xFF, 0xFFFF], {"care": False})
         self.assertRaises(RetryTask.MaxRetriesExceededError,
                           result.get)
         self.assertEqual(RetryTask.iterations, 3)
 
         RetryTask.max_retries = 1
         RetryTask.iterations = 0
-        result = RetryTask.apply([0xFF, 0xFFFF])
+        result = RetryTask.apply([0xFF, 0xFFFF], {"care": False})
         self.assertRaises(RetryTask.MaxRetriesExceededError,
                           result.get)
         self.assertEqual(RetryTask.iterations, 2)
@@ -320,6 +324,23 @@ class TestCeleryTasks(unittest.TestCase):
         publisher = t1.get_publisher()
         self.assertTrue(publisher.exchange)
 
+    def test_context_get(self):
+        request = self.createTaskCls("T1", "c.unittest.t.c.g").request
+        request.foo = 32
+        self.assertEqual(request.get("foo"), 32)
+        self.assertEqual(request.get("bar", 36), 36)
+
+    def test_task_class_repr(self):
+        task = self.createTaskCls("T1", "c.unittest.t.repr")
+        self.assertIn("class Task of", repr(task.app.Task))
+
+    def test_after_return(self):
+        task = self.createTaskCls("T1", "c.unittest.t.after_return")()
+        task.backend = Mock()
+        task.request.chord = 123
+        task.after_return("SUCCESS", 1.0, "foobar", (), {}, None)
+        task.backend.on_chord_part_return.assert_called_with(task)
+
     def test_send_task_sent_event(self):
         T1 = self.createTaskCls("T1", "c.unittest.t.t1")
         conn = T1.app.broker_connection()
@@ -493,30 +514,6 @@ class TestPeriodicTask(unittest.TestCase):
             MyPeriodic().remaining_estimate(datetime.now()),
             timedelta)
 
-    def test_timedelta_seconds_returns_0_on_negative_time(self):
-        delta = timedelta(days=-2)
-        self.assertEqual(MyPeriodic().timedelta_seconds(delta), 0)
-
-    def test_timedelta_seconds(self):
-        deltamap = ((timedelta(seconds=1), 1),
-                    (timedelta(seconds=27), 27),
-                    (timedelta(minutes=3), 3 * 60),
-                    (timedelta(hours=4), 4 * 60 * 60),
-                    (timedelta(days=3), 3 * 86400))
-        for delta, seconds in deltamap:
-            self.assertEqual(MyPeriodic().timedelta_seconds(delta), seconds)
-
-    def test_delta_resolution(self):
-        D = timeutils.delta_resolution
-
-        dt = datetime(2010, 3, 30, 11, 50, 58, 41065)
-        deltamap = ((timedelta(days=2), datetime(2010, 3, 30, 0, 0)),
-                    (timedelta(hours=2), datetime(2010, 3, 30, 11, 0)),
-                    (timedelta(minutes=2), datetime(2010, 3, 30, 11, 50)),
-                    (timedelta(seconds=2), dt))
-        for delta, shoulda in deltamap:
-            self.assertEqual(D(dt, delta), shoulda)
-
     def test_is_due_not_due(self):
         due, remaining = MyPeriodic().is_due(datetime.now())
         self.assertFalse(due)
