@@ -4,13 +4,13 @@ import smtplib
 try:
     from email.mime.text import MIMEText
 except ImportError:
-    from email.MIMEText import MIMEText
+    from email.MIMEText import MIMEText  # noqa
 
 supports_timeout = sys.version_info >= (2, 6)
 
 
 class SendmailWarning(UserWarning):
-    """Problem happened while sending the e-mail message."""
+    """Problem happened while sending the email message."""
 
 
 class Message(object):
@@ -27,7 +27,7 @@ class Message(object):
             self.to = [self.to]
 
     def __repr__(self):
-        return "<E-mail: To:%r Subject:%r>" % (self.to, self.subject)
+        return "<Email: To:%r Subject:%r>" % (self.to, self.subject)
 
     def __str__(self):
         msg = MIMEText(self.body, "plain", self.charset)
@@ -40,12 +40,13 @@ class Message(object):
 class Mailer(object):
 
     def __init__(self, host="localhost", port=0, user=None, password=None,
-            timeout=2):
+            timeout=2, use_ssl=False):
         self.host = host
         self.port = port
         self.user = user
         self.password = password
         self.timeout = timeout
+        self.use_ssl = use_ssl
 
     def send(self, message):
         if supports_timeout:
@@ -60,7 +61,10 @@ class Mailer(object):
                 socket.setdefaulttimeout(old_timeout)
 
     def _send(self, message, **kwargs):
-        client = smtplib.SMTP(self.host, self.port, **kwargs)
+        if (self.use_ssl):
+            client = smtplib.SMTP_SSL(self.host, self.port, **kwargs)
+        else:
+            client = smtplib.SMTP(self.host, self.port, **kwargs)
 
         if self.user and self.password:
             client.login(self.user, self.password)
