@@ -41,11 +41,11 @@ def process_initializer(app, hostname):
     """
     app = app_or_default(app)
     app.set_current()
-    [platforms.reset_signal(signal) for signal in WORKER_SIGRESET]
-    [platforms.ignore_signal(signal) for signal in WORKER_SIGIGNORE]
+    platforms.signals.reset(*WORKER_SIGRESET)
+    platforms.signals.ignore(*WORKER_SIGIGNORE)
     platforms.set_mp_process_title("celeryd", hostname=hostname)
 
-    # This is for windows and other platforms not supporting
+    # This is for Windows and other platforms not supporting
     # fork(). Note that init_worker makes sure it's only
     # run once per process.
     app.loader.init_worker()
@@ -190,6 +190,7 @@ class WorkController(object):
             # needs a custom implementation.
             self.eta_scheduler_cls = self.pool.Timer
 
+        self.autoscaler = None
         if autoscale:
             self.autoscaler = instantiate(self.autoscaler_cls, self.pool,
                                           max_concurrency=max_concurrency,
@@ -226,7 +227,8 @@ class WorkController(object):
                                     initial_prefetch_count=prefetch_count,
                                     pool=self.pool,
                                     priority_timer=self.priority_timer,
-                                    app=self.app)
+                                    app=self.app,
+                                    controller=self)
 
         # The order is important here;
         #   the first in the list is the first to start,
@@ -250,43 +252,42 @@ class WorkController(object):
                 blocking(component.start)
         except SystemTerminate:
             self.terminate()
-            raise SystemExit()
-        except (SystemExit, KeyboardInterrupt), exc:
+            raise
+        except:
             self.stop()
-            raise exc
+            raise
 
     def process_task(self, request):
         """Process task by sending it to the pool of workers."""
         try:
             request.task.execute(request, self.pool,
                                  self.loglevel, self.logfile)
-        except SystemTerminate:
-            self.terminate()
-            raise SystemExit()
-        except (SystemExit, KeyboardInterrupt), exc:
-            self.stop()
-            raise exc
         except Exception, exc:
             self.logger.critical("Internal error %s: %s\n%s" % (
                             exc.__class__, exc, traceback.format_exc()))
+        except SystemTerminate:
+            self.terminate()
+            raise SystemExit()
+        except BaseException, exc:
+            self.stop()
+            raise exc
 
     def stop(self, in_sighandler=False):
         """Graceful shutdown of the worker server."""
-        if in_sighandler and not self.pool.signal_safe:
-            return
-        blocking(self._shutdown, warm=True)
+        if not in_sighandler or self.pool.signal_safe:
+            blocking(self._shutdown, warm=True)
 
     def terminate(self, in_sighandler=False):
         """Not so graceful shutdown of the worker server."""
-        if in_sighandler and not self.pool.signal_safe:
-            return
-        blocking(self._shutdown, warm=False)
+        if not in_sighandler or self.pool.signal_safe:
+            blocking(self._shutdown, warm=False)
 
     def _shutdown(self, warm=True):
         what = (warm and "stopping" or "terminating").capitalize()
 
         if self._state != self.RUN or self._running != len(self.components):
             # Not fully started, can safely exit.
+            self._state = self.TERMINATE
             return
 
         self._state = self.CLOSE
@@ -297,9 +298,10 @@ class WorkController(object):
                     what, component.__class__.__name__))
             stop = component.stop
             if not warm:
-                stop = getattr(component, "terminate", stop)
+                stop = getattr(component, "terminate", None) or stop
             stop()
 
+        self.priority_timer.stop()
         self.consumer.close_connection()
         self._state = self.TERMINATE
 
