@@ -2,10 +2,11 @@ import os
 import platform
 import shelve
 
+from collections import defaultdict
+
 from kombu.utils import cached_property
 
 from celery import __version__
-from celery.utils.compat import defaultdict
 from celery.datastructures import LimitedSet
 
 #: Worker software/platform information.
@@ -50,7 +51,7 @@ def task_ready(request):
     reserved_requests.discard(request)
 
 
-if os.environ.get("CELERY_BENCH"):
+if os.environ.get("CELERY_BENCH"):  # pragma: no cover
     from time import time
 
     all_count = 0
@@ -59,13 +60,13 @@ if os.environ.get("CELERY_BENCH"):
     __reserved = task_reserved
     __ready = task_ready
 
-    def task_reserved(request):
+    def task_reserved(request):  # noqa
         global bench_start
         if bench_start is None:
             bench_start = time()
         return __reserved(request)
 
-    def task_ready(request):
+    def task_ready(request):  # noqa
         global all_count, bench_start
         all_count += 1
         if not all_count % bench_every:
